@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # Copyright 2010-2013 Joakim Möller
-# Copyright 2014 Marek Rudnicki
+# Copyright 2014-2015 Marek Rudnicki
 #
 # This file is part of matlab_wrapper.
 #
@@ -277,7 +277,7 @@ def load_engine_and_libs(matlab_root, options):
         if bits == '64bit':
             lib_dir = join(matlab_root, "bin", "glnxa64")
         else:
-            unsupported_platform(system, bits)
+            lib_dir = join(matlab_root, "bin", "glnx86")
 
         libeng = Library(
             join(lib_dir, 'libeng.so')
@@ -300,7 +300,7 @@ def load_engine_and_libs(matlab_root, options):
         if bits == '64bit':
             lib_dir = join(matlab_root, "bin", "win64")
         else:
-            unsupported_platform(system, bits)
+            lib_dir = join(matlab_root, "bin", "win32")
 
         if lib_dir not in os.environ['PATH']:
             os.environ['PATH'] = lib_dir + ';' + os.environ['PATH']
@@ -342,33 +342,16 @@ def load_engine_and_libs(matlab_root, options):
         version = tuple([int(v) for v in version_str.split('.')[:2]])
 
     except ValueError:
-        warnings.warn("Unable to identify MATLAB version, please let us know: https://github.com/mrkrd/matlab_wrapper")
+        warnings.warn("Unable to identify MATLAB (libeng) version.")
         version = None
 
 
-    if (system == 'Linux') and (version == (8,1)) and (bits == '64bit'):
-        pass
-
-    elif (system == 'Linux') and (version == (8,2)) and (bits == '64bit'):
-        pass
-
-    elif (system == 'Linux') and (version == (8,3)) and (bits == '64bit'):
+    if (system == 'Linux') and (version == (8,3)) and (bits == '64bit'):
         warnings.warn("You are using MATLAB version 8.3 (R2014a) on Linux, which appears to have a bug in engGetVariable().  You will only be able to use arrays of type double.")
 
-    elif (system == 'Linux') and (version == (8,4)) and (bits == '64bit'):
-        pass
-
-    elif (system == 'Windows') and (version == (8,3)) and (bits == '64bit'):
-        pass
-
-    elif (system == 'Darwin') and (version == (8,1)) and (bits == '64bit'):
-        pass
-
     elif (system == 'Darwin') and (version == (8,3)) and (bits == '64bit'):
         warnings.warn("You are using MATLAB version 8.3 (R2014a) on OS X, which appears to have a bug in engGetVariable().  You will only be able to use arrays of type double.")
 
-    else:
-        warnings.warn("Hi! You are using MATLAB version that was never tested with matlab_wrapper.  Please, let us know about that and visit out website <https://github.com/mrkrd/matlab_wrapper> or send me an email <marekrud@gmail.com>.  Your MATLAB version is {version} on {os} with {bits}.".format(version=version,os=system,bits=bits))
 
 
 
@@ -639,7 +622,7 @@ def mxarray_to_ndarray(libmx, pm):
             field_names.append(field_name)
 
         ### Get all fields
-        records = []
+        records = []            # [(x0, y0, z0), (x1, y1, z1), ... (xN, yN, zN)]
         for i in range(numelems):
             record = []
             for field_name in field_names:
@@ -654,28 +637,30 @@ def mxarray_to_ndarray(libmx, pm):
                 record.append(el)
             records.append(record)
 
-
         ### Set the dtypes right (if there is any ndarray, we want dtype=object)
-        arrays = zip(*records)
+        arrays = zip(*records)  # [(x0, x1, ... xN), (y0, y1, ... yN), (z0, z1, ... zN)]
         new_arrays = []
+
+        ## This loop is necessary, because np.rec.fromarrays() cannot
+        ## handle a list of arrays of the same size well
         for arr in arrays:
             contains_ndarray = np.any([isinstance(el, np.ndarray) for el in arr])
 
-            ## This loop is necessary, because np.rec.fromarrays()
-            ## cannot handle well a list of arrays of the same size
             if contains_ndarray:
                 newarr = np.empty(len(arr), dtype='O')
                 for i,a in enumerate(arr):
                     newarr[i] = a
-
             else:
                 newarr = np.array(arr)
 
             new_arrays.append(newarr)
 
-        out = np.rec.fromarrays(new_arrays, names=field_names)
-        out = out.reshape(dims[:ndims], order='F')
-        out = out.squeeze()
+        if new_arrays:
+            out = np.rec.fromarrays(new_arrays, names=field_names)
+            out = out.reshape(dims[:ndims], order='F')
+            out = out.squeeze()
+        else:
+            out = np.array([])
 
 
     else:
@@ -691,7 +676,7 @@ def ndarray_to_mxarray(libmx, arr):
 
     ### Prepare `arr` object (convert to ndarray if possible), assert
     ### data type
-    if isinstance(arr, str):
+    if isinstance(arr, str) or isinstance(arr, unicode):
         pass
 
     elif isinstance(arr, dict):
@@ -719,6 +704,9 @@ def ndarray_to_mxarray(libmx, arr):
     if isinstance(arr, str):
         pm = libmx.mxCreateString(arr)
 
+    elif isinstance(arr, unicode):
+        pm = libmx.mxCreateString(arr.encode('utf-8'))
+
     elif isinstance(arr, np.ndarray) and arr.dtype.kind in ['i','u','f','c']:
         dim = arr.ctypes.shape_as(mwSize)
         complex_flag = (arr.dtype.kind == 'c')
@@ -750,7 +738,7 @@ def ndarray_to_mxarray(libmx, arr):
         ctypes.memmove(mat_data, np_data, len(np_data))
 
 
-    elif isinstance(arr, np.ndarray) and arr.dtype.kind in ('O', 'S'):
+    elif isinstance(arr, np.ndarray) and arr.dtype.kind in ('O', 'S', 'U'):
         dim = arr.ctypes.shape_as(mwSize)
 
         pm = libmx.mxCreateCellArray(arr.ndim, dim)
